//*****************************************************
//*****************************************************
// This program solves toy model subject to uncertainty shock
//
//*************************************************************************
// Use Macro-Processors to define type j
@#define types = [ "1", "2", "3", "4", "5", "6","7", "8","9","10"]
//*************************************************************************
// Variables
var 
% endogenous variables   
    Y       (long_name='output')
    Vbar    (long_name='weighted average value') 
    W       (long_name='wage')
    w       (long_name='scaled wage')
    Z       (long_name='endogenous tfp')
    @#for type in types
    ehat_@{type}
    Y_@{type}
    logY_@{type}
    N_@{type}
    logN_@{type}
    Vbar_@{type}
    SALE_@{type}
    logVbar_@{type}
    @#endfor
    Y_9010  (long_name='output ratio 90/10')
    Y_7030  (long_name='output ratio 70/30')
    Y_10050 (long_name='output ratio 100/50')
    SALE_9010  (long_name='sales ratio 90/10')
    SALE_7030  (long_name='sales ratio 70/30')
    SALE_10050 (long_name='sales ratio 100/50')
    N_9010  (long_name='labor ratio 90/10')
    N_7030  (long_name='labor ratio 70/30')
    N_10050 (long_name='labor ratio 100/50')
    Vbar_9010  (long_name='value ratio 90/10')
    Vbar_7030  (long_name='value ratio 70/30')
    Vbar_10050 (long_name='value ratio 100/50')
% log-variables
    logY
    logW
    logw
    logVbar
    logZ
    logA
    logY_9010
    logY_7030
    logY_10050
    logN_9010
    logN_7030
    logN_10050
    logVbar_9010
    logVbar_7030
    logVbar_10050
    logSALE_9010
    logSALE_7030
    logSALE_10050
% exogenous variables
    rho
    logrho
    theta
    logtheta
    N
    logN
    A
    psigma
    pmu
    ;
varexo erho etheta eN eNews eA eSigmaE;

// Parameters
parameters 
% model parameters
    pbeta             % discount factor
    psigmass          % sigma of lognormal dist.
    pthetass          % ss financial shock
    prhoss            % ss uncertainty shock
    pNss   
    pAss   
% idiosyncractic state distribution
    pngridz           % # of discretized states of z;
    muz               % mean of logz
    rhoz              % persistence of z
    sigz              % volatility of z
    @#for type in types
    z_@{type}
    pi_@{type} 
    @#endfor
% aggregate state distribution
    prhorho   
    psigmarho 
    prhotheta    
    psigmatheta
    prhoN
    psigmaN
    psigmaNews
    prhoA
    psigmaA 
    prhoSigmaE
    psigmaSigmaE
    ;

// Parameter Initial Values
% parameters of functional form: quarterly frequency
pbeta    = 0.96;                            % discount factor
psigmass = 0.80;                            % s.d. of log-normal dist.
pthetass = 0.35;                            % ss financial shock
prhoss   = 0.82;                            % ss uncertainty shock
pNss     = 0.33;
pAss     = 1;
% discretize AR(1) process of z using Tauchen method
pngridz  = 10;                              % # of states for z;
muz      = 0;
rhoz     = 0.80;  
sigz     = 0.25;
% aggregate state distribution          
prhorho     = 0.84;   
psigmarho   = 0.01;
prhotheta   = 0.90;     
psigmatheta = 0.01;     
prhoN       = 0.90;     
psigmaN     = 0.01;   
psigmaNews  = 0.01;
prhoA       = 0.90;
psigmaA     = 0.01;
prhoSigmaE  = 0.90;
psigmaSigmaE= 0.01;  

//*************************************************************************
// Model
model;
    %--------------------------------------------------------------------------
    % idiosyncratic cut-off
    @#for type in types 
    ehat_@{type} = W/(A*z_@{type});
    @#endfor

    %--------------------------------------------------------------------------
    % idiosyncratic value function
    @#for type in types 
    Vbar_@{type} = (1+theta*((1./ehat_@{type})*exp(pmu+0.5*psigma^2)*normcdf((pmu+psigma^2-log(ehat_@{type}))/psigma)-1+ normcdf((log(ehat_@{type})-pmu)/psigma)))*pbeta*(rho*Vbar_@{type}(+1)+(1-rho)*Vbar(+1));
    @#endfor

    %--------------------------------------------------------------------------
    % idiosyncratic value function in log
    @#for type in types
    logVbar_@{type} = log(Vbar_@{type}); 
    @#endfor

    %--------------------------------------------------------------------------
    % aggregate value function
    Vbar = 1*(
    @#for i in types
    +pi_@{i}*Vbar_@{i}
    @#endfor
    );

    %--------------------------------------------------------------------------
    % idiosyncratic labor demand function
    @#for type in types
    N_@{type} = theta/W*pbeta*(rho*Vbar_@{type}(+1)+(1-rho)*Vbar(+1))*(1-normcdf((log(ehat_@{type})-pmu)/psigma)); 
    @#endfor

    %--------------------------------------------------------------------------
    % idiosyncratic labor demand function in log
    @#for type in types
    logN_@{type} = log(N_@{type}); 
    @#endfor

    %--------------------------------------------------------------------------
    % aggregate labor demand function
    N = 1*(
    @#for i in types
    +pi_@{i}*N_@{i}
    @#endfor
    );

    %--------------------------------------------------------------------------
    % idiosyncratic output function
    @#for type in types
    SALE_@{type} = (theta*pbeta*(rho*Vbar_@{type}(+1)+(1-rho)*Vbar(+1)))/ehat_@{type}*exp(pmu+0.5*psigma^2)*normcdf((pmu+psigma^2-log(ehat_@{type}))/psigma);
    @#endfor

    %--------------------------------------------------------------------------
    % idiosyncratic output function
    @#for type in types
    Y_@{type} = A*z_@{type}*N_@{type};
    @#endfor

    %--------------------------------------------------------------------------
    % idiosyncratic output function in log
    @#for type in types
    logY_@{type} = log(Y_@{type}); 
    @#endfor

    %--------------------------------------------------------------------------
    % aggregate output function
    Y = 1*(
    @#for i in types
    +pi_@{i}*Y_@{i}
    @#endfor
    );

    %--------------------------------------------------------------------------
    % endogenous TFP function
    Z = Y/(A*N); 

    %--------------------------------------------------------------------------
    % output, value and labor ratios
    w        = W/A;
    Y_9010   = Y_9 / Y_1;
    Y_7030   = Y_7 / Y_3;
    Y_10050  = Y_10/ Y_5;
    N_9010   = N_9 / N_1;
    N_7030   = N_7 / N_3;
    N_10050  = N_10/ N_5;
    Vbar_9010  = Vbar_9 / Vbar_1;
    Vbar_7030  = Vbar_7 / Vbar_3;
    Vbar_10050 = Vbar_10/ Vbar_5;
    SALE_9010  = SALE_9 / SALE_1;
    SALE_7030  = SALE_7 / SALE_3;
    SALE_10050 = SALE_10/ SALE_5;
    %--------------------------------------------------------------------------
    % log variables definition functions
    logrho   = log(rho);
    logtheta = log(theta);
    logY     = log(Y);
    logN     = log(N);
    logW     = log(W);
    logw     = log(w);
    logZ     = log(Z);
    logA     = log(A);
    logVbar  = log(Vbar);
    logY_9010  = log(Y_9010);
    logY_7030  = log(Y_7030);
    logY_10050 = log(Y_10050);
    logN_9010  = log(N_9010);
    logN_7030  = log(N_7030);
    logN_10050 = log(N_10050);
    logVbar_7030  = log(Vbar_7030);
    logVbar_9010  = log(Vbar_9010);
    logVbar_10050 = log(Vbar_10050);
    logSALE_7030  = log(SALE_7030);
    logSALE_9010  = log(SALE_9010);
    logSALE_10050 = log(SALE_10050);

    %--------------------------------------------------------------------------
    % shock process
    logrho   = (1-prhorho)*log(prhoss) + prhorho*logrho(-1) - psigmarho*erho;
    logtheta = (1-prhotheta)*log(pthetass) + prhotheta*logtheta(-1) - psigmatheta*etheta;
    log(N)   = (1-prhoN)*log(pNss)+ prhoN*log(N(-1)) + psigmaN*eN;
    log(A)   = (1-prhoA)*log(pAss)+ prhoA*log(A(-1)) - psigmaA*eA - psigmaNews*eNews(-4);
    log(psigma) = (1-prhoSigmaE)*log(psigmass)+ prhoSigmaE*log(psigma(-1))+ psigmaSigmaE*eSigmaE;
    pmu = -0.5*psigma^2;
end;

//*************************************************************************
// Shock section
shocks;
var erho   = 1;
var etheta = 1;
var eN     = 1; 
var eNews  = 1; 
var eA     = 1; 
var eSigmaE= 1; 
end;

steady;
check;

// Simulation
options_.pruning=1;
stoch_simul(order=3,pruning,k_order_solver,noprint,nograph,irf=40);

IRF_periods =40;
burnin= 1000; %periods for convergence

% simulation with no shock to get stochastic steady state
zero_shock_mat     = zeros(burnin+IRF_periods,M_.exo_nbr);   
IRF_zero_shock_mat = simult_(M_,options_,oo_.dr.ys,oo_.dr,zero_shock_mat,options_.order)'; 
Stoch_SS           = IRF_zero_shock_mat(1+burnin,:);         

% simulation with turbulence shock
rho_shock_mat      = zeros(burnin+IRF_periods,M_.exo_nbr);
rho_shock_mat(burnin+1,strmatch('erho',M_.exo_names,'exact'))= 1;
IRF_rho_shock_mat  = simult_(M_,options_,oo_.dr.ys,oo_.dr,rho_shock_mat,options_.order)';
IRF_rho_shock_pc_mat  = (IRF_rho_shock_mat(1+burnin+1:1+burnin+IRF_periods,:)-IRF_zero_shock_mat(1+burnin+1:1+burnin+IRF_periods,:))./repmat(Stoch_SS,IRF_periods,1); 

% simulation with uncertainty shock
SigmaE_shock_mat      = zeros(burnin+IRF_periods,M_.exo_nbr);
SigmaE_shock_mat(burnin+1,strmatch('eSigmaE',M_.exo_names,'exact'))= 1;
IRF_SigmaE_shock_mat  = simult_(M_,options_,oo_.dr.ys,oo_.dr,SigmaE_shock_mat,options_.order)';
IRF_SigmaE_shock_pc_mat  = (IRF_SigmaE_shock_mat(1+burnin+1:1+burnin+IRF_periods,:)-IRF_zero_shock_mat(1+burnin+1:1+burnin+IRF_periods,:))./repmat(Stoch_SS,IRF_periods,1); 

Y_pos 	    = strmatch('Y',M_.endo_names,'exact');
N_pos 	    = strmatch('N',M_.endo_names,'exact');
W_pos 	    = strmatch('W',M_.endo_names,'exact');
Vbar_pos    = strmatch('Vbar',M_.endo_names,'exact');
Y_10_pos    = strmatch('Y_10',M_.endo_names,'exact');
V_7030_pos  = strmatch('V_7030',M_.endo_names,'exact');
N_7030_pos  = strmatch('N_7030',M_.endo_names,'exact');
Z_pos 	    = strmatch('Z',M_.endo_names,'exact');
rho_pos 	= strmatch('rho',M_.endo_names,'exact');
psigma_pos 	= strmatch('psigma',M_.endo_names,'exact');

