function [ys,params,check] = TOY_Dynamics_steadystate(ys,exo,M_,options_)
%%%%%%%%%%%%%%%%%%%%%%%%%
% This function is a subroutine of dynare that solves steady states of RBC.

%%%%%%%%%%%%%%%%%%%%%%%%%

%% Read out parameters to access them with their name
NumberOfParameters = M_.param_nbr;
for ii = 1:NumberOfParameters
  paramname = M_.param_names{ii};
  eval([ paramname ' = M_.params(' int2str(ii) ');']);
end

check = 0;

%% Enter model equations here
%--------------------------------------------------------------------------
% exogenous shock variables
rho   = prhoss;
theta = pthetass;
N     = pNss;
A     = pAss;
psigma= psigmass;
pmu   = -0.5*psigma^2;

% discretize AR(1) process of z using Tauchen method
[logz, Pi]  = tauchen(muz,rhoz,sigz,pngridz);
z           = exp(logz);         % grid of z
pi          = limitdist(Pi)';
%--------------------------------------------------------------------------
% steady state wage: Wss
W     = fzero(@(W) 1 -pi*gfun(W,pbeta,pmu,psigma,rho,theta,z),6);    
%--------------------------------------------------------------------------
% steady state aggregate firm value: Vbar
Vbar  = fzero(@(Vbar) N -pi*nfun(Vbar,pbeta,pmu,psigma,rho,theta, W,z),15);
%--------------------------------------------------------------------------
% steady state idiosyncratic cut-off
ehat  = W./z;
%--------------------------------------------------------------------------
% steady state idiosyncratic labor
n     = nfun(Vbar,pbeta,pmu,psigma,rho, theta, W, z);
logn  = log(n);
%--------------------------------------------------------------------------
% steady state idiosyncratic output
y     = max(yfun(Vbar,pbeta,pmu,psigma,rho,theta,W,z),eps);
logy  = log(y);
%--------------------------------------------------------------------------
% steady state idiosyncratic sales
sale  = max(salefun(Vbar,pbeta,pmu,psigma,rho,theta,W,z),eps);
%--------------------------------------------------------------------------
% steady state idiosyncratic value
v     = gfun(W,pbeta,pmu,psigma,rho,theta,z).*Vbar;
logv  = log(v);
%--------------------------------------------------------------------------
% steady state aggregate output
Y     = pi*y;
%--------------------------------------------------------------------------
% steady state aggregate TFP
Z     = Y/(A*N);

for type = 1:pngridz
eval(sprintf('z_%d = z(type)', type));
eval(sprintf('pi_%d = pi(type)', type));
eval(sprintf('ehat_%d = ehat(type)', type));
eval(sprintf('Vbar_%d = v(type)', type));
eval(sprintf('N_%d = n(type)', type));
eval(sprintf('Y_%d = y(type)', type));
eval(sprintf('SALE_%d = sale(type)', type));
eval(sprintf('logY_%d = logy(type)', type));
eval(sprintf('logN_%d = logn(type)', type));
eval(sprintf('logVbar_%d = logv(type)', type));
end
clc;
%--------------------------------------------------------------------------
% other defined variables
w        = W/A;
Y_9010   = Y_9 / Y_1;
Y_7030   = Y_7 / Y_3;
Y_10050  = Y_10/ Y_5;
N_9010   = N_9 / N_1;
N_7030   = N_7 / N_3;
N_10050  = N_10/ N_5;
Vbar_9010  = Vbar_9 / Vbar_1;
Vbar_7030  = Vbar_7 / Vbar_3;
Vbar_10050 = Vbar_10/ Vbar_5;
SALE_9010  = SALE_9 / SALE_1;
SALE_7030  = SALE_7 / SALE_3;
SALE_10050 = SALE_10/ SALE_5;
logrho   = log(rho);
logtheta = log(theta);
logY     = log(Y);
logN     = log(N);
logW     = log(W);
logw     = log(w);
logVbar  = log(Vbar);
logZ     = log(Z);
logA     = log(A);
logY_9010  = log(Y_9010);
logY_7030  = log(Y_7030);
logY_10050 = log(Y_10050);
logN_9010  = log(N_9010);
logN_7030  = log(N_7030);
logN_10050 = log(N_10050);
logVbar_7030  = log(Vbar_7030);
logVbar_9010  = log(Vbar_9010);
logVbar_10050 = log(Vbar_10050);
logSALE_7030  = log(SALE_7030);
logSALE_9010  = log(SALE_9010);
logSALE_10050 = log(SALE_10050);

%% End own model equations

params=NaN(NumberOfParameters,1);
for iter = 1:length(M_.params) %update parameters set in the file
  eval([ 'params(' num2str(iter) ') = ' M_.param_names{iter} ';' ])
end

NumberOfEndogenousVariables = M_.orig_endo_nbr; %auxiliary variables are set automatically
for ii = 1:NumberOfEndogenousVariables
  varname = M_.endo_names{ii};
  eval(['ys(' int2str(ii) ') = ' varname ';']);
end
